const { USER_ROLES } = require('enums').COMMON;
const { BOUNDARY_ZONES } = require('./constants');
const Env = require('utils/env_variables');


function getZonePrototypesBlackList(role) {
    switch (role) {
        case USER_ROLES.PROJECT_USER:
        case USER_ROLES.PROJECT_OPERATIONAL_ADMIN:
        case USER_ROLES.PROJECT_SYSTEM_ADMIN: {
            return [BOUNDARY_ZONES.BOUNDARY_OUT, BOUNDARY_ZONES.BOUNDARY_IN];
        }
        case USER_ROLES.COMPANY_ADMIN:
        default: {
            return [];
        }
    }
}

function filterZonesByUserRole(user, zones) {
    if (user && user.role && Array.isArray(zones)) {
        const black_list = getZonePrototypesBlackList(user.role);
        return zones.filter(zone => !black_list.includes(zone.zone_type_name));
    }
    return zones;
}

function getRouterPathBasedOnEnv(path) {
    return Env.isTestEnv() ? `addon/${path}` : path;
}

function isBoundaryZone(zone_type_name) {
    return zone_type_name === BOUNDARY_ZONES.BOUNDARY_IN || zone_type_name === BOUNDARY_ZONES.BOUNDARY_OUT;
}

function isCoupledZone(zone) {
    return !!(zone && zone.user_params && zone.user_params.coupled);
}

function isCoupledBoundaryZone(zone) {
    return !!(zone && isBoundaryZone(zone.zone_type_name) && isCoupledZone(zone));
}

function getCoupleId(zone) {
    return zone && zone.user_params && zone.user_params.coupled;
}

function isNeedReCouple(updatedZone, zoneBeforeUpdate) {
    if (!updatedZone || !zoneBeforeUpdate) {
        return false;
    }
    const isTheSameZone = updatedZone.uid === zoneBeforeUpdate.uid;
    const isCoupleIdChange = getCoupleId(zoneBeforeUpdate) !== getCoupleId(updatedZone);
    
    return isTheSameZone && isCoupledBoundaryZone(updatedZone) && isCoupledBoundaryZone(zoneBeforeUpdate) && isCoupleIdChange;
}

module.exports = {
    isCoupledZone,
    isBoundaryZone,
    isNeedReCouple,
    isCoupledBoundaryZone,
    filterZonesByUserRole,
    getRouterPathBasedOnEnv,
    getZonePrototypesBlackList,
};
